from fractalgen import generate_julia
import numpy as np

def test_julia_output():
    c = complex(-0.4, 0.6)
    result = generate_julia(c, width=120, height=90, max_iter=60)
    assert isinstance(result, np.ndarray)
    assert result.shape == (120, 90)
    assert result.max() <= 60



import matplotlib.pyplot as plt

def generate_sierpinski(order, p1, p2, p3):
    if order == 0:
        return [(p1, p2, p3)]
    else:
        mid12 = midpoint(p1, p2)
        mid23 = midpoint(p2, p3)
        mid31 = midpoint(p3, p1)

        return (
            generate_sierpinski(order - 1, p1, mid12, mid31) +
            generate_sierpinski(order - 1, p2, mid23, mid12) +
            generate_sierpinski(order - 1, p3, mid31, mid23)
        )

def midpoint(p1, p2):
    return ((p1[0]+p2[0])/2, (p1[1]+p2[1])/2)

def plot_sierpinski(triangles, show=True, save_path=None):
    plt.figure(figsize=(8, 8))
    for tri in triangles:
        x, y = zip(*(tri + (tri[0],)))  # Close the triangle
        plt.plot(x, y, 'k-')
    plt.axis('equal')
    plt.axis('off')
    plt.title("Sierpinski Triangle")
    if save_path:
        plt.savefig(save_path, dpi=150)
        print(f"Saved Sierpinski Triangle to {save_path}")
    if show:
        plt.show()



import matplotlib.pyplot as plt
import math

def generate_koch(order, length=1.0):
    def koch_curve(p1, p2, depth):
        if depth == 0:
            return [p1, p2]
        else:
            dx = (p2[0] - p1[0]) / 3
            dy = (p2[1] - p1[1]) / 3
            pA = (p1[0] + dx, p1[1] + dy)
            pB = (p1[0] + 2*dx, p1[1] + 2*dy)
            angle = math.atan2(dy, dx) - math.pi / 3
            px = pA[0] + math.cos(angle) * math.hypot(dx, dy)
            py = pA[1] + math.sin(angle) * math.hypot(dx, dy)
            pC = (px, py)
            return (
                koch_curve(p1, pA, depth - 1)[:-1] +
                koch_curve(pA, pC, depth - 1)[:-1] +
                koch_curve(pC, pB, depth - 1)[:-1] +
                koch_curve(pB, p2, depth - 1)
            )

    p1 = (0, 0)
    p2 = (length, 0)
    p3 = (length / 2, math.sqrt(3) * length / 2)

    side1 = koch_curve(p1, p2, order)
    side2 = koch_curve(p2, p3, order)
    side3 = koch_curve(p3, p1, order)
    return side1 + side2 + side3 + [p1]

def plot_koch(points, show=True, save_path=None):
    x, y = zip(*points)
    plt.figure(figsize=(8, 6))
    plt.plot(x, y)
    plt.axis('equal')
    plt.title("Koch Snowflake")
    if save_path:
        plt.savefig(save_path, dpi=150)
        print(f"Saved Koch Snowflake to {save_path}")
    if show:
        plt.show()



from fractalgen import generate_sierpinski

def test_sierpinski_triangle_count():
    p1 = (0, 0)
    p2 = (1, 0)
    p3 = (0.5, 0.866)
    triangles = generate_sierpinski(order=3, p1=p1, p2=p2, p3=p3)
    
    # For order n, there are 3^n triangles
    assert len(triangles) == 3**3



import numpy as np
import matplotlib.pyplot as plt

def generate_julia(c, xmin=-2, xmax=2, ymin=-2, ymax=2, width=800, height=800, max_iter=100):
    x = np.linspace(xmin, xmax, width)
    y = np.linspace(ymin, ymax, height)
    Z = x[:, None] + 1j * y[None, :]
    div_time = np.full(Z.shape, max_iter)

    for i in range(max_iter):
        Z = Z**2 + c
        mask = (np.abs(Z) > 2) & (div_time == max_iter)
        div_time[mask] = i
        Z[mask] = 2

    return div_time

def plot_julia(div_time, extent=[-2, 2, -2, 2], show=True, save_path=None):
    plt.imshow(div_time.T, cmap='magma', extent=extent)
    plt.colorbar(label="Escape time")
    plt.title("Julia Set")
    if save_path:
        plt.savefig(save_path, dpi=150)
        print(f"Saved Julia Set to {save_path}")
    if show:
        plt.show()



import numpy as np
import matplotlib.pyplot as plt

def generate_mandelbrot(xmin=-2.0, xmax=1.0, ymin=-1.5, ymax=1.5,
                        width=800, height=800, max_iter=100):
    x = np.linspace(xmin, xmax, width)
    y = np.linspace(ymin, ymax, height)
    C = x[:, None] + 1j * y[None, :]
    Z = np.zeros_like(C, dtype=complex)
    div_time = np.full(C.shape, max_iter)

    for i in range(max_iter):
        Z = Z**2 + C
        diverged = np.abs(Z) > 2
        div_now = diverged & (div_time == max_iter)
        div_time[div_now] = i
        Z[diverged] = 2

    return div_time

def plot_mandelbrot(div_time, extent=[-2, 1, -1.5, 1.5], show=True, save_path=None):
    """
    Plot the Mandelbrot fractal.

    Args:
        div_time (ndarray): 2D array of divergence times from generate_mandelbrot.
        extent (list): Plotting extent for imshow [xmin, xmax, ymin, ymax].
        show (bool): If True, display the image interactively using plt.show().
        save_path (str or None): If set, save the plot to this file path.
    """
    plt.imshow(div_time.T, cmap='inferno', extent=extent)
    plt.colorbar(label='Iteration count')
    plt.title("Mandelbrot Set")
    plt.tight_layout()

    if save_path:
        plt.savefig(save_path, dpi=150)
        print(f"Saved Mandelbrot image as {save_path}")

    if show:
        try:
            plt.show()
        except Exception as e:
            print(f"Could not display plot: {e}")


import matplotlib.pyplot as plt
import math

def generate_lsystem(axiom, rules, iterations):
    for _ in range(iterations):
        axiom = ''.join([rules.get(c, c) for c in axiom])
    return axiom

def draw_lsystem(instructions, angle=25, length=5, show=True, save_path=None):
    x, y = 0, 0
    angle_rad = math.radians(angle)
    direction = math.pi / 2
    stack = []
    points = [(x, y)]

    for cmd in instructions:
        if cmd == 'F':
            x += length * math.cos(direction)
            y += length * math.sin(direction)
            points.append((x, y))
        elif cmd == '+':
            direction += angle_rad
        elif cmd == '-':
            direction -= angle_rad
        elif cmd == '[':
            stack.append((x, y, direction))
        elif cmd == ']':
            x, y, direction = stack.pop()
            points.append((x, y))

    xs, ys = zip(*points)
    plt.figure(figsize=(8, 8))
    plt.plot(xs, ys)
    plt.axis('equal')
    plt.axis('off')
    plt.title("L-System Fractal")
    if save_path:
        plt.savefig(save_path, dpi=150)
        print(f"Saved L-system fractal to {save_path}")
    if show:
        plt.show()



from fractalgen import generate_lsystem

def test_lsystem_length_increases():
    axiom = "F"
    rules = {"F": "F+F-F"}
    
    string_1 = generate_lsystem(axiom, rules, iterations=1)
    string_2 = generate_lsystem(axiom, rules, iterations=2)
    
    assert isinstance(string_1, str)
    assert len(string_2) > len(string_1)
    assert "F" in string_2



from fractalgen import generate_koch

def test_koch_point_count():
    # For order 0, it's a triangle: 3 sides + 1 closing point
    points = generate_koch(order=0)
    assert len(points) >= 4

    # For higher order, point count should increase
    points_3 = generate_koch(order=3)
    assert len(points_3) > len(points)



from fractalgen import generate_mandelbrot
import numpy as np

def test_mandelbrot_shape():
    result = generate_mandelbrot(width=100, height=80, max_iter=50)
    assert isinstance(result, np.ndarray)
    assert result.shape == (100, 80)
    assert result.dtype == np.int32 or result.dtype == np.int64
    assert result.max() <= 50



from .mandelbrot import generate_mandelbrot, plot_mandelbrot
from .koch import generate_koch, plot_koch
from .sierpinski import generate_sierpinski, plot_sierpinski
from .julia import generate_julia, plot_julia
from .lsystem import generate_lsystem, draw_lsystem

__all__ = [
    "generate_mandelbrot", "plot_mandelbrot",
    "generate_koch", "plot_koch",
    "generate_sierpinski", "plot_sierpinski",
    "generate_julia", "plot_julia",
    "generate_lsystem", "draw_lsystem"
]



from setuptools import setup, find_packages

setup(
    name="fractalgen",
    version="0.1.0",
    description="A package to generate fractal patterns in Python",
    author="Your Name",
    packages=find_packages(),
    install_requires=["numpy", "matplotlib"],
    python_requires=">=3.7",
)
